!----------------------------------------------------------------------
!----------------------------------------------------------------------
! Copyright (C) 2003, 2004, 2005
!      Chaok Seok, Evangelos Coutsias, Matthew Jacobson, and Ken Dill.
!      UCSF, Univeristy of New Mexico, Seoul National University
                                                                                                                         
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
!----------------------------------------------------------------------------
! written by Chaok Seok, 2003
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
MODULE in_out
!----------------------------------------------------------------------------
  integer, parameter :: dp_io = kind(1.0d0)
!----------------------------------------------------------------------------
CONTAINS
!-----------------------------------------------------------------------
subroutine read_pdb_backbone(pdb_name, res_name, r_n, r_a, r_c, stt_res, end_res)
!-----------------------------------------------------------------------
! read backbone atom coord from a pdb file
!-----------------------------------------------------------------------
  implicit none
  character(len=100), intent(in) :: pdb_name
  integer, intent(in) ::  stt_res, end_res
  character(len=4), intent(out) :: res_name(:)
  real(dp_io), intent(out) :: r_n(:,:), r_a(:,:), r_c(:,:)
  character(len=10) :: string, atmname, resname
  integer :: res_no, unit=10, ioerror, ir
  real(dp_io) :: x, y, z

!-----------------------------------------------------------------------

  open(unit, file = trim(pdb_name))
  do    ! exit at ATOM
     read(unit, 50) string
     if (string == 'ATOM') exit
  end do
  backspace(unit)

  do    ! exit if not ATOM
     read(unit, 50) string
     if (string /= 'ATOM') exit
     backspace(unit)
     read(unit, 56, iostat=ioerror) atmname, resname, res_no, x, y, z
     if (ioerror < 0) exit 
     if (res_no > end_res) exit
     if (res_no >= stt_res) then
        ir = res_no - stt_res + 1
        res_name(ir) = resname
!        print*, ir, atmname
        if (atmname == 'N') r_n(:,ir) = (/ x, y, z /)
        if (atmname == 'CA') r_a(:,ir) = (/ x, y, z /)
        if (atmname == 'C')  r_c(:,ir) = (/ x, y, z /)
     end if
  end do
  close(unit)

50 format (a6)
56 format (13x, a2, 2x, a3, 2x, i4, 4x, 3f8.3)

end subroutine read_pdb_backbone
!-----------------------------------------------------------------------
subroutine write_pdb_backbone(pdb_name, res_name, r_n, r_a, r_c, stt_res, end_res)
!-----------------------------------------------------------------------
! write backbone atom coord in pdb format
!-----------------------------------------------------------------------
  implicit none
  character(len=100), intent(in) :: pdb_name
  integer, intent(in) :: stt_res, end_res
  character(len=4), intent(in) :: res_name(:)
  real(dp_io), intent(in) :: r_n(:,:), r_a(:,:), r_c(:,:)
  integer :: i, j, k, unit = 20, res_no, ir

  open(unit, file = trim(pdb_name))

  k = 0
  do res_no = stt_res, end_res
     ir = res_no - stt_res + 1
     k = k+1
     write(unit,72) 'ATOM', k, 'N  ', res_name(ir), res_no, r_n(1:3,ir)
     k = k+1
     write(unit,72) 'ATOM', k, 'CA ', res_name(ir), res_no, r_a(1:3,ir)
     k = k+1
     write(unit,72) 'ATOM', k, 'C  ', res_name(ir), res_no, r_c(1:3,ir)
  end do
  write(unit, "(a3)") 'END'
  close (unit)

72 format(a4, 3x, i4, 2x, a3, 1x, a3, 2x, i4, 4x, 3f8.3)

end subroutine write_pdb_backbone
!-----------------------------------------------------------------------
END MODULE in_out
!-----------------------------------------------------------------------
